{% extends "base.html" %}

{% block page_meta %}
	<meta name="title" content="{{ blog.title }} | Home">
	<meta name="description" content="{{ blog.description }}">
	<!-- Google will often use this as its description of your page/site. Make it good. -->

	<meta name="author" content="{{ blog.author }}">
	<meta name="Copyright" content="{{ blog.copyright }}">

	<!-- Dublin Core Metadata : http://dublincore.org/ -->
	<meta name="DC.title" content="{{ blog.title }} | Home">
	<meta name="DC.subject" content="{{ blog.description }}">
	<meta name="DC.creator" content="{{ blog.author }}">
{% endblock page_meta %}

{% block content %}
	{% for p in index %}
		<h3><a href="{{p.meta.url}}">{% if p.title %}{{ p.title }}{% else %}#{% endif %}</a></h3>
		<p class="muted"><small>
			posted on {{ p.meta.date.strftime("%d/%m/%Y %H:%M")  }}
			{% if p.meta.tags %}
				{% for t in p.meta.tags %}<a href="{{ blog.url }}tag/{{ t }}/">#{{ t }}</a> {% endfor %}
			{% endif %}
		</small></p>
		{{ p.html|safe() }}
		<hr />
	{% endfor %}
{% endblock %}
